/* rectick.c: tty record output for backtick */
/* $Id: rectick.c,v 1.6 2003/11/25 08:35:11 knok Exp $ */

/*
Copyright (C) 2003 NOKUBI Takatsugu <knok@daionet.gr.jp>
This is free software with ABSOLUTELY NO WARRANTY.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA
 */

#include <stdio.h>
#include <string.h>
#include <getopt.h>

#include "gnet.h"

int settermraw(void);
void resetterm(void);
void sighandler(int);
void putbuf();
void rotbuf(char);

#define BUF 1024
int buflen = 8;
char *buf;
char *sockpath = "/tmp/backtick";

int main(int argc, char *argv[])
{
  GUnixSocket * gs, *gc;
  GIOChannel *io;
  for (;;) {
    int c = getopt(argc, argv, "l:");
    if (c == -1)
      break;
    switch (c) {
    case 'l':
      buflen = atoi(optarg);
      break;
    }
  }
  if ((argc - 1) >= optind) {
    sockpath = argv[optind];
  }
  if (buflen <= 0)
    buflen = 1;
  buf = calloc(1, buflen);
  gs = gnet_unix_socket_server_new(sockpath);
  gc = gnet_unix_socket_server_accept(gs);
  io = gnet_unix_socket_get_io_channel(gc);
  putbuf();
  while (1) {
      GIOError ge;
    char tmp[BUF];
    gsize readp;
    tmp[0] = 'X';
    ge = gnet_io_channel_readline(io, tmp, (BUF-1), &readp);
    if (ge != G_IO_ERROR_NONE)
	break;
    if (readp == 0)
	break;
    rotbuf(tmp[0]);
    putbuf();
  }
  return 0;
}

void putbuf()
{
  int i;
  for (i = buflen - 1; i >= 0; i --) {
    if (buf[i] != 0)
      switch (buf[i]) {
      case 0x0a:
	printf("[LF] ");
	break;
      case 0x0d:
	printf("[CR] ");
	break;
      case 0x1b:
	printf("[ESC] ");
	break;
      default:
	printf("[%c] ", buf[i]);
      }
  }
  printf("\n");
  fflush(stdout);
}

void rotbuf(char c)
{
  int i;
  for (i = buflen - 1; i >= 0; i --) {
    buf[i + 1] = buf[i];
  }
  buf[0] = c;
}
